{ Demo program for ChroMathsome - Linear Algebra.
  Below, you find example implementations of using functions available in
  ChroMathsome - Linear Algebra.

  In this demonstration program we will use matrices to calculate
  the coefficients of a curve that fits pairs of data points
  aka. linear/polynomial regression.

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Since ChroMathsome - Linear Algebra automatically updates the dimensions of
  matrices and vectors (if needed), you can give most of the matrices/vectors in the
  initialization part (InitRealMatrix, ...) a dummy value.
  You are, however, encouraged to enter the correct values (if you know them at forehand)
  since this will make your code much more readable!

  Read the help file for information about the implemented functions
  and their limitations/possibilities!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: January 30, 1998
  Reader may do with this code whatever he/she wants.

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "S".              }
unit Stats;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  LinAlg, LinAlgP, Menus, StdCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    CurveFit1: TMenuItem;
    StraightLine1: TMenuItem;
    Polynom1: TMenuItem;
    Memo1: TMemo;
    procedure StraightLine1Click(Sender: TObject);
    procedure Polynom1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ Linear regression.
  Fit the points (0, 1), (1, 3), (2, 4), (3, 4) into a straight line.
  The least squares solution Y = M * V.
  V gives the coefficients a and b for the line Y = a*X + b.
  where V is given by
           Transpose(M) * Y
       V = ----------------
           Transpose(M) * M
                                                                         }
procedure TForm1.StraightLine1Click(Sender: TObject);
var M, Y, V, M1: RealMatrix;
begin
     InitRealMatrix(M, 4, 2);
     InitRealMatrix(Y, 4, 1);
     InitRealMatrix(V, 2, 1);
     InitRealMatrix(M1, 1, 1);
     try
     Y.Matrix^[1]^[1]:= 1;
     Y.Matrix^[2]^[1]:= 3;
     Y.Matrix^[3]^[1]:= 4;
     Y.Matrix^[4]^[1]:= 4;

     M.Matrix^[1]^[1]:= 0;
     M.Matrix^[1]^[2]:= 1;
     M.Matrix^[2]^[1]:= 1;
     M.Matrix^[2]^[2]:= 1;
     M.Matrix^[3]^[1]:= 2;
     M.Matrix^[3]^[2]:= 1;
     M.Matrix^[4]^[1]:= 3;
     M.Matrix^[4]^[2]:= 1;

     { Find matrix V = [Transpose(M)*Y]/[Transpose(M)*M]. }
     Copy_RealMatrix(M1, M);
     RealMatrix_Transpose(M1);
     RealMatrix_Multiplication(V, M1, M);  { Transpose(M)*M }
     Copy_RealMatrix(M, V);
     RealMatrix_Inverse(M); { A/B = A*Inverse(B) }
     RealMatrix_Multiplication(V, M1, Y);  { Transpose(M)*Y }
     RealMatrix_Multiplication(M1, M, V);

     Memo1.Lines.Add('Coefficients of straight line:');
     Memo1.Lines.Add('Coefficient a :' + FloatToStr(M1.Matrix^[1]^[1]));
     Memo1.Lines.Add('Coefficient b :' + FloatToStr(M1.Matrix^[2]^[1]));
     Memo1.Lines.Add('-> straight line Y = a*X + b');

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(Y);
     DestroyRealMatrix(V);
     DestroyRealMatrix(M1);
     end;
end;

{ Polynomial regression.
  According to Newton's 2nd Law of Motion, a body near the earth's surface falls vertically downward
  according to the equation: y = y0 + v0*t + 1/2*g*t (1)
  where
  y = vertical displacement at time t
  y0 = initial vertical displacement at time t0 = 0
  v0 = initial velocity at time t0 = 0
  g = Newton's constant of acceleration of gravity near the earth's surface.

  We wish to evaluate g experimentally.
  A weight is released with unknown initial displacement and velocity.

  At a fixed time interval the distance fallen from a fixed reference point is measured.
  The following results are obtained:
  at times t = 0.1, 0.2, ..., 0.5 seconds the weight has fallen
  y = -0.055, 0.094, 0.314, 0.756 and 1.138 meters, resp., from the reference point.
  Calculate g using this data.

  Solution: fit the quadratic curve y = a + b*X + c*X^2 to the five data points.
            The least squares solution is given by Y = M * V
            where
            Y = [ y1 y2 y3 y4 y5 ]
            M = [ 1 t1 t1^2 ]
                [ 1 t2 t2^2 ]
                [ ... ... . ]
                [ 1 t5 t5^2 ]
            V = [ a b c ]
            
       where V is given by
           Transpose(M) * Y
       V = ----------------
           Transpose(M) * M

  To find g, compare the resulting equation of the parabole with equation (1).
  (for those who do not know this: g ~= 9.81)                                              }
procedure TForm1.Polynom1Click(Sender: TObject);
var M, Y, V, M1: RealMatrix;
begin
     InitRealMatrix(M, 5, 3);
     InitRealMatrix(Y, 5, 1);
     InitRealMatrix(V, 3, 1);
     InitRealMatrix(M1, 1, 1);
     try
     Y.Matrix^[1]^[1]:= -0.055;
     Y.Matrix^[2]^[1]:= 0.094;
     Y.Matrix^[3]^[1]:= 0.314;
     Y.Matrix^[4]^[1]:= 0.756;
     Y.Matrix^[5]^[1]:= 1.138;

     M.Matrix^[1]^[1]:= 1;
     M.Matrix^[1]^[2]:= 0.1;
     M.Matrix^[1]^[3]:= 0.1*0.1;
     M.Matrix^[2]^[1]:= 1;
     M.Matrix^[2]^[2]:= 0.2;
     M.Matrix^[2]^[3]:= 0.2*0.2;
     M.Matrix^[3]^[1]:= 1;
     M.Matrix^[3]^[2]:= 0.3;
     M.Matrix^[3]^[3]:= 0.3*0.3;
     M.Matrix^[4]^[1]:= 1;
     M.Matrix^[4]^[2]:= 0.4;
     M.Matrix^[4]^[3]:= 0.4*0.4;
     M.Matrix^[5]^[1]:= 1;
     M.Matrix^[5]^[2]:= 0.5;
     M.Matrix^[5]^[3]:= 0.5*0.5;

     { Find matrix V = [Transpose(M)*Y]/[Transpose(M)*M]. }
     Copy_RealMatrix(M1, M);
     RealMatrix_Transpose(M1);
     RealMatrix_Multiplication(V, M1, M);  { Transpose(M)*M }
     Copy_RealMatrix(M, V);
     RealMatrix_Inverse(M); { A/B = A*Inverse(B) }
     RealMatrix_Multiplication(V, M1, Y);  { Transpose(M)*Y }
     RealMatrix_Multiplication(M1, M, V);

     Memo1.Lines.Add('Coefficients of polynom:');
     Memo1.Lines.Add('Coefficient a: ' + FloatToStr(M1.Matrix^[1]^[1]));
     Memo1.Lines.Add('Coefficient b: ' + FloatToStr(M1.Matrix^[2]^[1]));
     Memo1.Lines.Add('Coefficient c: ' + FloatToStr(M1.Matrix^[3]^[1]));
     Memo1.Lines.Add('-> polynom Y = a + b*X + c*X^2');
     Memo1.Lines.Add('The experimental value for g = ' + FloatToStr(2*M1.Matrix^[3]^[1]));

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(Y);
     DestroyRealMatrix(V);
     DestroyRealMatrix(M1);
     end;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

end.
